# Send eMail (even through SSL) from FHEM without any external programs
# Perl code based on http://robertmaldon.blogspot.de/2006/10/sending-email-through-google-smtp-from.html
# Alexey Ozerov (C) 2015 GPL

# You need an SMTP server access and you have to enter SMTP server config in this file

# Requires latest SMTP/SSL perl libraries
# apt-get install libnet-smtp-ssl-perl
# cpan install Net::SSLeay
# cpan install IO::Socket::SSL
# cpan install Authen::SASL
# cpan install Net::SMTP::SSL

# Send test email from FHEM with { send_mail ('your@email.de','Test FHEM','Test');; }

package main;
use strict;
use warnings;
use POSIX;
use Net::SMTP::SSL;
use MIME::Base64;
use File::Spec;
use LWP::MediaTypes;

sub sendmail_Initialize($$)
{
  my ($hash) = @_;
}

### SMTP Server Config ###

my $smtp_from     = 'server@ozerov.de';
my $smtp_user     = 'server@ozerov.de';
my $smtp_password = '55gTAf6Db9';
my $smtp_server   = 'www.ozerov.de';
my $smtp_port     = 465;
my $smtp_debug    = 0;


### send_mail() ###

sub send_mail {
my $to = $_[0];
my $subject = $_[1];
my $body = $_[2];

my $smtp;

if (not $smtp = Net::SMTP::SSL->new($smtp_server, Port => $smtp_port, Debug => $smtp_debug)) 
{ Log (1, "Could not connect to SMTP server $smtp_server"); return; }

# Figure out current date/time for the message date stamp
# Date stamp format is DD Monthname YY HH:MM:SS TIMEZONE
my @monthnames = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
my @now = gmtime(time);
for (0..$#now) {
     # Make single-digit date/time elements two digits 
     if (length($now[$_]) lt 2) {$now[$_] = '0'.$now[$_];}  # (i.e. prefix with '0')
}

# Slice off just the time and date elements we need from the output of gmtime()
my($YY, $MON, $DD, $HH, $MM, $SS) = @now[5,4,3,2,1,0];      
$YY += 1900;                               # gmtime() epoch starts at 1900

my $monthname = $monthnames[$MON];         # Get the name of the month

# Finally build the silly date stamp!
my $datestring = "Date: $DD $monthname $YY $HH:$MM:$SS GMT";


if (not $smtp->auth($smtp_user, $smtp_password))
{ Log (1, "SMTP authentication failed for $smtp_user"); return; }

$smtp->mail($smtp_from . "\n");
my @recepients = split(/,/, $to);
foreach my $recp (@recepients) {
    $smtp->to($recp . "\n");
}
$smtp->data();
$smtp->datasend("$datestring\n");
$smtp->datasend("From: " . $smtp_from . "\n");
$smtp->datasend("To: " . $to . "\n");
$smtp->datasend("Subject: " . $subject . "\n");
$smtp->datasend("\n");
$smtp->datasend($body . "\n");
$smtp->dataend();
$smtp->quit;
}


### send_mail_with_attachments() ###

sub send_mail_with_attachments {
 my $to = shift(@_);
 my $subject = shift(@_);
 my $body = shift(@_);
 my @attachments = @_;

 my $smtp;

 if (not $smtp = Net::SMTP::SSL->new($smtp_server, Port => $smtp_port, Debug => $smtp_debug)) 
 { Log (1, "Could not connect to SMTP server $smtp_server"); return; }

 # Authenticate
 if (not $smtp->auth($smtp_user, $smtp_password))
 { Log (1, "SMTP authentication failed for $smtp_user"); return; }

 # Create arbitrary boundary text used to seperate
 # different parts of the message
 my ($bi, $bn, @bchrs);
 my $boundry = "";
 foreach $bn (48..57,65..90,97..122) {
     $bchrs[$bi++] = chr($bn);
 }
 foreach $bn (0..20) {
     $boundry .= $bchrs[rand($bi)];
 }

 # Send the header
 $smtp->mail($smtp_from . "\n");
 my @recepients = split(/,/, $to);
 foreach my $recp (@recepients) {
     $smtp->to($recp . "\n");
 }
 $smtp->data();
 $smtp->datasend("Date: " .giveMeDate('MDYHMS') . "\n");
 $smtp->datasend("From: " . $smtp_from . "\n");
 $smtp->datasend("To: " . $to . "\n");
 $smtp->datasend("Subject: " . $subject . "\n");
 $smtp->datasend("MIME-Version: 1.0\n");
 $smtp->datasend("Content-Type: multipart/mixed; BOUNDARY=\"$boundry\"\n");

 # Send the body
 $smtp->datasend("\n--$boundry\n");
 $smtp->datasend("Content-Type: text/plain\n");
 $smtp->datasend($body . "\n\n");

 # Send attachments
 foreach my $file (@attachments) {
     unless (-f $file) {
         return ("Unable to find attachment file $file\n");
         next;
     }
     my($bytesread, $buffer, $data, $total);
     open(FH, "$file") || return ("Failed to open $file\n");
     binmode(FH);
     while (($bytesread = sysread(FH, $buffer, 1024)) == 1024) {
         $total += $bytesread;
         $data .= $buffer;
     }
     if ($bytesread) {
         $data .= $buffer;
         $total += $bytesread;
     }
     close FH;

     # Get the file name without its directory
     my ($volume, $dir, $fileName) = File::Spec->splitpath($file);
  
     # Try and guess the MIME type from the file extension so
     # that the email client doesn't have to
     my $contentType = guess_media_type($file);
  
     if ($data) {
         $smtp->datasend("--$boundry\n");
         $smtp->datasend("Content-Type: $contentType; name=\"$fileName\"\n");
         $smtp->datasend("Content-Transfer-Encoding: base64\n");
         $smtp->datasend("Content-Disposition: attachment; =filename=\"$fileName\"\n\n");
         $smtp->datasend(encode_base64($data));
         $smtp->datasend("--$boundry\n");
     }
 }

 # Quit
 $smtp->datasend("\n--$boundry--\n"); # send boundary end message
 $smtp->datasend("\n");
 $smtp->dataend();
 $smtp->quit;
}



1;